/*
 *  Author:    Alberto Gil Tesa
 *  Website:   http://giltesa.com
 *  License:   CC BY-NC-SA 3.0
 *
 *  Project:   RTC Dallas DS1307
 *  File:      gDS1307.cpp
 *  Date:      24/09/2013
 *  Version:   1.2
 */


#include "gDS1307.h"


/**
 * LA CLASE TIEMPO
 * Nos proporciona todas las propiedades necesarias para almacenar los parametros del RTC,
 * bien para ponerlo en hora o para obtener sus datos.
 *
 * El constructor pone a "0" todas las propiedades al instanciar un objeto.
 */
Tiempo :: Tiempo()
{
    segundos   = 0;
    minutos    = 0;
    hora       = 0;
    hora24h    = 0;
    diaSemana  = 1;
    diaMes     = 1;
    mes        = 1;
    anio       = 2013;
}



/**
 * LA CLASE gDS1307
 * Se encarga de poner en hora o de obtener los datos del reloj.
 *
 * El constructor inicia la comunicacion con el bus i2c
 */
gDS1307 :: gDS1307()
{
    Wire.begin();
}



/**
 * LA CLASE gDS1307 - METODO setFecha
 * Permite ajustar la fecha del DS1307, necesita para ello 4 parametros: dia de la semana, dia del mes, mes y el ao.
 * El dia de la semana y el mes pueden ser representados tanto en numero como por escrito en mayusculas y sin acentos.
 * Si alguno de esos parametros estuviera en un rango incorrecto prevalecera el que tenga configurado el DS1307 en ese momento.
 */
void gDS1307 :: setFecha( uint8_t diaSemana, uint8_t diaMes, uint8_t mes, uint16_t anio )
{
    Tiempo t = this->getDatos();

    Wire.beginTransmission(DS1307_ADDRESS);

    #if ARDUINO >= 100
        Wire.write(0);
        Wire.write(decToBcd(t.segundos));
        Wire.write(decToBcd(t.minutos));
        Wire.write(decToBcd(t.hora24h));
        Wire.write((diaSemana >= 1 && diaSemana <= 7) ? decToBcd(diaSemana) : decToBcd(t.diaSemana));
        Wire.write((diaMes >= 1 && diaMes <= 31) ? decToBcd(diaMes) : decToBcd(t.diaMes));
        Wire.write((mes >= 1 && mes <= 12) ? decToBcd(mes) : decToBcd(t.mes));

        if( anio >= 0  &&  anio <= 99 )
            Wire.write(decToBcd(anio));
        else if( anio >= 2000  &&  anio <= 2099 )
            Wire.write(decToBcd(anio-2000));
        else
            Wire.write(decToBcd(t.anio));
    #else
        Wire.send(0);
        Wire.send(decToBcd(t.segundos));
        Wire.send(decToBcd(t.minutos));
        Wire.send(decToBcd(t.hora24h));
        Wire.send((diaSemana >= 1 && diaSemana <= 7) ? decToBcd(diaSemana) : decToBcd(t.diaSemana));
        Wire.send((diaMes >= 1 && diaMes <= 31) ? decToBcd(diaMes) : decToBcd(t.diaMes));
        Wire.send((mes >= 1 && mes <= 12) ? decToBcd(mes) : decToBcd(t.mes));

        if( anio >= 0  &&  anio <= 99 )
            Wire.send(decToBcd(anio));
        else if( anio >= 2000  &&  anio <= 2099 )
            Wire.send(decToBcd(anio-2000));
        else
            Wire.send(decToBcd(t.anio));
    #endif

    Wire.endTransmission();
}



/**
 * LA CLASE gDS1307 - METODO setTiempo
 * Permite ajustar la fecha del DS1307, necesita para ello 3 parametros: La hora en formato PM/24h, los minutos y los segundos.
 * Si alguno de esos parametros estuviera en un rango incorrecto prevalecera el que tenga configurado el DS1307 en ese momento.
 */
void gDS1307 :: setTiempo( uint8_t hora24h, uint8_t minutos, uint8_t segundos )
{
    Tiempo t = this->getDatos();

    Wire.beginTransmission(DS1307_ADDRESS);

    #if ARDUINO >= 100
        Wire.write(0);
        Wire.write((segundos >= 0 && segundos <= 59) ? decToBcd(segundos) : decToBcd(t.segundos));
        Wire.write((minutos >= 0 && minutos <= 59) ? decToBcd(minutos) : decToBcd(t.minutos));
        Wire.write((hora24h >= 0 && hora24h <=23) ? decToBcd(hora24h) : decToBcd(t.hora24h));
        Wire.write(decToBcd(t.diaSemana));
        Wire.write(decToBcd(t.diaMes));
        Wire.write(decToBcd(t.mes));
        Wire.write(decToBcd(t.anio-2000));
    #else
        Wire.send(0);
        Wire.send((segundos >= 0 && segundos <= 59) ? decToBcd(segundos) : decToBcd(t.segundos));
        Wire.send((minutos >= 0 && minutos <= 59) ? decToBcd(minutos) : decToBcd(t.minutos));
        Wire.send((hora24h >= 0 && hora24h <=23) ? decToBcd(hora24h) : decToBcd(t.hora24h));
        Wire.send(decToBcd(t.diaSemana));
        Wire.send(decToBcd(t.diaMes));
        Wire.send(decToBcd(t.mes));
        Wire.send(decToBcd(t.anio-2000));
    #endif

    Wire.endTransmission();
}



/**
 * LA CLASE gDS1307 - METODO getDatos
 * Permite obtener todos los datos del DS1307, estos datos son devueltos por valor.
 */
Tiempo gDS1307 :: getDatos()
{
    Tiempo t;

    Wire.beginTransmission(DS1307_ADDRESS);
    #if ARDUINO >= 100
        Wire.write(0);
    #else
        Wire.send(0);
    #endif
    Wire.endTransmission();
    Wire.requestFrom(DS1307_ADDRESS, 7);

    #if ARDUINO >= 100
        t.segundos   = bcdToDec(Wire.read() & 0x7f);
        t.minutos    = bcdToDec(Wire.read());
        t.hora24h    = bcdToDec(Wire.read() & 0x3f);
        t.hora       = (t.hora24h > 12) ? t.hora24h - 12 : t.hora24h;
        t.diaSemana  = bcdToDec(Wire.read());
        t.diaMes     = bcdToDec(Wire.read());
        t.mes        = bcdToDec(Wire.read());
        t.anio       = bcdToDec(Wire.read()) + 2000;
    #else
        t.segundos   = bcdToDec(Wire.receive() & 0x7f);
        t.minutos    = bcdToDec(Wire.receive());
        t.hora24h    = bcdToDec(Wire.receive() & 0x3f);
        t.hora       = (t.hora24h > 12) ? t.hora24h - 12 : t.hora24h;
        t.diaSemana  = bcdToDec(Wire.receive());
        t.diaMes     = bcdToDec(Wire.receive());
        t.mes        = bcdToDec(Wire.receive());
        t.anio       = bcdToDec(Wire.receive()) + 2000;
    #endif

    return t;
}



/**
 * Pasa de Decimal a BCD
 */
uint8_t gDS1307 :: decToBcd(uint8_t val)
{
  return( (val/10*16) + (val%10) );
}



/**
 * Pasa pe BCD a Decimal
 */
uint8_t gDS1307 :: bcdToDec(uint8_t val)
{
  return( (val/16*10) + (val%16) );
}